import { defineStore } from "pinia";
import { ref } from "vue";
import {
  imagesWeatherDay,
  imagesWeatherNight,
  orderWeather,
} from "@/constants";

const useWeatherDataStore = defineStore("weatherData", () => {
  const weatherStateText = ref("N/A");
  const weatherStateImage = ref(null);
  const actualWeather = ref("Clear");
  const temperature = ref("N/A");
  const humidity = ref("N/A");
  const actualNighttime = ref(false);
  const dateTime = ref("N/A");

  const subscribers = ref([]);
  const subscribeToWeather = () => {
    window.require &&
      window.requirejs(["baja!"], (baja) => {
        const subscriber = new baja.Subscriber(); // 初始化 subscriber
        subscriber.attach("changed", (prop) => {
          // 根據 slotName 判斷資料類型
          if (prop.$slotName === "temp") {
            temperature.value = prop.$getValue().getValueDisplay();
          } else if (prop.$slotName === "humidity") {
            humidity.value = prop.$getValue().getValueDisplay();
          } else if (prop.$slotName === "state") {
            actualWeather.value = prop.$getValue().getValueDisplay();
            loadWeather(actualWeather.value); // 載入天氣資料
          } else if (prop.$slotName === "sunDown") {
            actualNighttime.value = !prop.$getValue().getValueDisplay();
            loadWeather(actualWeather.value); // 重新載入天氣資料
          }
        });

        // 訂閱 Niagara 資料
        baja.Ord.make(
          "station:|slot:/Services/WeatherService/WeatherService/WeatherService/current"
        )
          .get({ subscriber })
          .then((result) => {
            console.log("Successfuly subscribed to weather data", result);
          })
          .catch((err) => {
            console.error(`訂閱 weather 失敗: ${err.message}`);
            subscriber.detach("changed"); // 發生錯誤時取消訂閱
          });
        subscribers.value.push(subscriber);
      });
  };

  // 載入天氣資料函數
  const loadWeather = (actualWeather) => {
    let weatherStateTextTemp = "Unknown";
    let weatherStateImageTemp = null;

    // 找到 actualWeather 在 orderWeather 中的索引
    const actualWeatherIndex = orderWeather.indexOf(
      actualWeather.replace(/\s/g, "")
    );

    if (actualWeatherIndex === -1) {
      // 如果找不到，表示 weather 狀態不在 orderWeather 中
      console.error(
        "天氣狀態不在 orderWeather 陣列中！實際值：" + actualWeather
      );
      weatherStateTextTemp = "Unknown";
      weatherStateImageTemp = imagesWeatherDay[weatherStateTextTemp];
    } else if (actualWeather === "none") {
      console.warn("未設定天氣狀態或點不存在");
      weatherStateTextTemp = "Unknown";
      weatherStateImageTemp = imagesWeatherDay[weatherStateTextTemp];
    } else {
      if (actualNighttime.value) {
        console.log("Nighttime activ");
        weatherStateTextTemp = orderWeather[actualWeatherIndex];
        weatherStateImageTemp = imagesWeatherNight[weatherStateTextTemp];
      } else {
        console.log("Daytime activ");
        weatherStateTextTemp = orderWeather[actualWeatherIndex];
        weatherStateImageTemp = imagesWeatherDay[weatherStateTextTemp];
      }
    }

    weatherStateImage.value = weatherStateImageTemp;
    weatherStateText.value = weatherStateTextTemp; // 更新天氣狀態
  };

  const updateTime = () => {
    const date = new Date();
    window.require &&
      window.requirejs(["baja!"], (baja) => {
        const bAbsTime = baja.AbsTime.make({ jsDate: date });
        bAbsTime
          .toDateTimeString()
          .then((dateTimeStr) => {
            dateTime.value = dateTimeStr;
          })
          .catch((error) => {
            console.error("轉換時間字串失敗:", error);
          });
      });
  };

  const clearAllSubscriber = () => {
    subscribers.value.forEach((subscriber) => {
      subscriber.detach("changed");
    });
    subscribers.value = [];
  };

  return {
    weatherStateText,
    weatherStateImage,
    actualWeather,
    temperature,
    humidity,
    actualNighttime,
    dateTime,
    subscribeToWeather,
    updateTime,
    clearAllSubscriber,
  };
});

export default useWeatherDataStore;
